﻿using Hotline.Share.Dtos.File;
using Hotline.Share.Dtos.FlowEngine.Workflow;
using Hotline.Share.Dtos.Hotspots;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Enums.Order;
using Hotline.Share.Enums.Settings;
using Hotline.Share.Requests;
using XF.Utility.EnumExtensions;

namespace Hotline.Share.Dtos.Order
{
    public class OrderDto : UpdateOrderDto
    {
        #region 工单属性

        /// <summary>
        /// 工单状态
        /// </summary>
        public EOrderStatus Status { get; set; }
        public string StatusText => Status.GetDescription();

        /// <summary>
        /// 过期状态
        /// </summary>
        public EExpiredStatus? ExpiredStatus => CalculateExpiredState();

        /// <summary>
        /// 过期状态
        /// </summary>
        public string ExpiredStatusText => ExpiredStatus?.GetDescription() ?? string.Empty;

        /// <summary>
        /// 处理方式（直办、交办）
        /// </summary>
        public EProcessType ProcessType { get; set; }

        ///// <summary>
        ///// 进展情况
        ///// </summary>
        //public EProgress Progress { get; set; }

        /// <summary>
        /// 是否公开
        /// </summary>
        public bool IsPublicity { get; set; }

        /// <summary>
        /// 工单编码（20220101000001）
        /// </summary>
        public string No { get; set; }

        /// <summary>
		/// 是否110来的工单
		/// </summary>
		public bool IsNo110 { get; set; }

        public string? TagNames { get; set; }

        #region 流程信息

        /// <summary>
        /// 工单开始时间(受理/接办时间=流程开启时间)
        /// </summary>
        public DateTime? StartTime { get; set; }

        /// <summary>
        /// 交办时间(中心交部门办理时间)
        /// </summary>
        public DateTime? CenterToOrgTime { get; set; }

        /// <summary>
        /// 归档时间（暂为流程结束时间，因流程结束自动归档）
        /// </summary>
        public DateTime? FiledTime { get; set; }


        /// <summary>
        /// 归档意见
        /// </summary>
        public string? FileOpinion { get; set; }

        /// <summary>
        /// 办结时长（分钟）
        /// 办结时间-交办时间
        /// </summary>
        public double HandleDuration { get; set; }

        /// <summary>
        /// 办结工作日时长(分钟)
        /// </summary>
        public double HandleDurationWorkday { get; set; }

        /// <summary>
        /// 全流程时长（分钟）
        /// 归档时间-创建时间
        /// </summary>
        public double AllDuration { get; set; }

        #region 实际办理信息（节点，部门，意见）

        /// <summary>
        /// 实际办理节点code（会签状态此字段保存最外层会签发起节点code）
        /// </summary>
        public string? ActualHandleStepCode { get; set; }

        /// <summary>
        /// 实际办理节点名称（会签状态此字段保存最外层会签发起节点名称）
        /// </summary>
        public string? ActualHandleStepName { get; set; }

        /// <summary>
        /// 到达实际办理节点时间（stepBox创建时间）
        /// </summary>
        public DateTime? ActualHandleStepCreateTime { get; set; }

        /// <summary>
        /// 实际办理节点签收时间
        /// </summary>
        public DateTime? ActualHandleStepAcceptTime { get; set; }

        /// <summary>
        /// 实际办理时间
        /// </summary>
        public DateTime? ActualHandleTime { get; set; }

        /// <summary>
        /// 实际办理人id
        /// </summary>
        public string? ActualHandlerId { get; set; }

        /// <summary>
        /// 实际办理人名称
        /// </summary>
        public string? ActualHandlerName { get; set; }

        /// <summary>
        /// 实际办理部门名称
        /// </summary>
        public string? ActualHandleOrgName { get; set; }

        /// <summary>
        /// 实际办理部门编码
        /// </summary>
        public string? ActualHandleOrgCode { get; set; }

        /// <summary>
        /// 实际办理部门行政区划编码
        /// </summary>
        public string? ActualHandleOrgAreaCode { get; set; }

        /// <summary>
        /// 实际办理部门行政区划名称
        /// </summary>
        public string? ActualHandleOrgAreaName { get; set; }

        /// <summary>
        /// 实际办理对象(会签停留在顶级发起节点)
        /// 办理时选择的办理对象，可能是部门也可能是用户
        /// </summary>
        public string? ActualHandlerValue { get; set; }
        public string? ActualHandlerKey { get; set; }
        public EHandlerType? ActualHandlerType { get; set; }

        /// <summary>
        /// 实际办理意见（办理中...or 最终办理意见）
        /// </summary>
        public string ActualOpinion { get; set; } = "办理中...";

        /// <summary>
        /// 真实办理人姓名（手动填写）
        /// </summary>
        public string? RealHandlerName { get; set; }

        /// <summary>
        /// 真实办理人电话（手动填写）
        /// </summary>
        public string? RealHandlerPhone { get; set; }

        /// <summary>
        /// 沟通方式（手动填写）
        /// </summary>
        public ERealCommunicationMode? RealCommunicationMode { get; set; }

        /// <summary>
        /// 沟通时间（手动填写）
        /// </summary>
        public DateTime? RealCommunicationTime { get; set; }

        /// <summary>
        /// 沟通地点（手动填写）
        /// </summary>
        public string? RealCommunicationAddress { get; set; }

        /// <summary>
        /// 已与市民沟通
        /// </summary>
        public bool? RealIsContacted { get; set; }

        /// <summary>
        /// 已与市民现场沟通
        /// </summary>
        public bool? RealContactLocale { get; set; }

        #endregion

        #region 一级部门

        /// <summary>
        /// 一级部门code
        /// </summary>
        public string? OrgLevelOneCode { get; set; }

        /// <summary>
        /// 一级部门名称
        /// </summary>
        public string? OrgLevelOneName { get; set; }

        #endregion

        #region 受理人(开启流程的话务员)

        /// <summary>
        /// 受理人id
        /// </summary>
        public string? AcceptorId { get; set; }

        /// <summary>
        /// 受理人名称
        /// </summary>
        public string? AcceptorName { get; set; }

        /// <summary>
        /// 受理人工号
        /// </summary>
        public string? AcceptorStaffNo { get; set; }

        /// <summary>
        /// 受理人部门编码
        /// </summary>
        public string? AcceptorOrgCode { get; set; }

        /// <summary>
        /// 受理人部门名称
        /// </summary>
        public string? AcceptorOrgName { get; set; }

        #endregion

        /// <summary>
        /// 会签类型
        /// </summary>
        public ECounterSignType? CounterSignType { get; set; }
        public string? CounterSignTypeText => CounterSignType?.GetDescription() ?? "未会签";

        #region 签收

        /// <summary>
        /// 签收人id
        /// </summary>
        public string? SignerId { get; set; }

        /// <summary>
        /// 签收人名称
        /// </summary>
        public string? SignerName { get; set; }

        #endregion

        #endregion


        #endregion

        public DateTime CreationTime { get; set; }


        public string? ContactMask { get; set; }

        public string SmsSendedText => SmsSended ? "已发送" : "未发送";

        /// <summary>
        /// 市民查询密码
        /// </summary>
        public string? Password { get; set; }

        //public string? HotspotCodeLv1 { get; set; }

        public string WorkflowId { get; set; }
        public WorkflowDto Workflow { get; set; }

        /// <summary>
        /// 来电/信人性别
        /// </summary>
        public string FromGenderText => FromGender.GetDescription();

        /// <summary>
        /// 来电/信人身份
        /// </summary>
        public string IdentityTypeText => IdentityType?.GetDescription();

        ///// <summary>
        ///// 受理类型
        ///// </summary>
        ////public string AcceptTypeText => AcceptType.GetDescription();

        /// <summary>
        /// 紧急程度
        /// </summary>
        public string EmergencyLevelText => EmergencyLevel.GetDescription();

        ///// <summary>
        ///// 超期时间描述（需求：超期3天、0.5天后超期）
        ///// </summary>
        //public string ExpiredText => CalculateExpiredText();

        /// <summary>
        /// 是否可编辑
        /// </summary>
        public bool CanEdit => !string.IsNullOrEmpty(SignerId) &&
                               (Status is EOrderStatus.WaitForAccept or EOrderStatus.BackToUnAccept or EOrderStatus.SpecialToUnAccept);

        /// <summary>
        /// 是否可签收
        /// </summary>
        public bool CanSign => string.IsNullOrEmpty(SignerId);

        /// <summary>
        /// 待结束会签编号
        /// </summary>
        public string? CountersignId { get; set; }

        /// <summary>
        /// 是否可以延期
        /// </summary>
        public bool IsCanDelay { get; set; }

        /// <summary>
        /// 是否可以取消延期
        /// </summary>
        public bool IsCanCancelDelay { get; set; }

        /// <summary>
        /// 最后一次延期天数
        /// </summary>
        public string? DelayString { get; set; }

        /// <summary>
        /// 是否可退回
        /// </summary>
        public bool CanPrevious { get; set; }

        /// <summary>
        /// 是否可办理
        /// </summary>
        public bool CanHandle { get; set; }

        /// <summary>
        /// 延期申请数量
        /// </summary>
        public int DelayingCount => OrderDelays?.Count ?? 0;

        /// <summary>
        /// 是否紧急
        /// </summary>
        public bool IsUrgent { get; set; }

        public string IsUrgentText => IsUrgent ? "紧急" : "";

		/// <summary>
		/// 发布范围
		/// </summary>
		public bool? PublishState { get; set; }

        /// <summary>
        /// 是否延期 
        /// </summary>
        public string? DelayText => DelayingCount > 0 ? "是" : "否";

        private string CalculateExpiredText()
        {
            //todo 完整处理方案：1.创建待过期表，Id,过期时间，即将过期时间等字段。2.延迟消息通知处理过期，删除子表数据，处理order过期状态字段。3.此处即可采用expiredStatus进行判断
            //todo 目前暂时采用过期时间计算
            if (!ExpiredTime.HasValue)
                return string.Empty;
            var days = (DateTime.Now - ExpiredTime.Value).TotalDays;
            switch (days)
            {
                case > 0:
                    return $"超期{days:N1}天";
                case < 0:
                    return $"{days:N1}天后超期";
                default:
                    return "超期不到1天";
            }
        }

        public EExpiredStatus? CalculateExpiredState()
        {
            DateTime? dateTime = DateTime.Now;
            if (Status >= EOrderStatus.Filed)
            {
                dateTime = ActualHandleTime;
            }
            //ExpiredStatus
            if (ExpiredTime.HasValue)
            {
                if (dateTime < NearlyExpiredTime)
                {
                    return EExpiredStatus.Normal;
                }
                else if (dateTime > NearlyExpiredTime && dateTime < ExpiredTime)
                {
                    return EExpiredStatus.GoingToExpired;
                }
                else
                {
                    return EExpiredStatus.Expired;
                }
            }
            return null;
        }

        public OrderExtensionDto OrderExtension { get; set; }

        public HotspotDto? Hotspot { get; set; }

        /// <summary>
        /// 延期
        /// </summary>
        public List<OrderDelayDto> OrderDelays { get; set; }

        /// <summary>
        /// 甄别
        /// </summary>
        public List<OrderScreenListDto> OrderScreens { get; set; }

        public List<RepeatableEventDetailOpDto> RepeatableEventDetails { get; set; } = new();

        public List<OrderVisitDto> OrderVisits { get; set; }

        public List<FileJson>? FileJson { get; set; }

        /// <summary>
        /// 可直接访问的通话录音地址
        /// </summary>
        public string? RecordingBaseAddress { get; set; }
        public string? RecordingAbsolutePath { get; set; }

        public int ReTransactNum { get; set; }

        /// <summary>
        /// 敏感标签
        /// </summary>
        public List<string>? Sensitive { get; set; }

        public string? SensitiveText => (Sensitive != null && Sensitive.Any()) ? string.Join(',', Sensitive) : "";

        public string IsProvinceText => IsProvince ? "省工单" : "市工单";

        /// <summary>
        /// 部门是否解决
        /// </summary>
        public bool IsResolved { get; set; }
        public string IsResolvedText => IsResolved ? "已解决" : "未解决";

        #region 副本工单

        /// <summary>
        /// 修改人	
        /// </summary>
        public string AuditUserName { get; set; }
        public string AuditUserId { get; set; }

        /// <summary>
        ///  修改时间
        /// </summary>
        public DateTime AuditTime { get; set; }
        #endregion

        public string OverDays => GetOverDays();


        public string GetOverDays()
        {
            var overDays = "0天";
            if (ExpiredTime.HasValue)
            {
                if (FiledTime.HasValue && ExpiredTime < FiledTime)
                {
                    overDays = (FiledTime - ExpiredTime).Value.Days + "天";
                }
                else if (ExpiredTime < DateTime.Now)
                {
                    overDays = (DateTime.Now - ExpiredTime).Value.Days + "天";
                }
            }
            return overDays;
        }
    }

    public class UpdateOrderDto : AddOrderDto
    {
        public string Id { get; set; }
    }

    public class AddOrderDto : Position
    {
        #region 来电信息

        /// <summary>
        /// 来源渠道
        /// </summary>
        public string? SourceChannel { get; set; }
        public string? SourceChannelCode { get; set; }

        /// <summary>
        /// 渠道为电话时，此字段存在
        /// </summary>
        public string? CallId { get; set; }

        /// <summary>
        /// 来电归属地
        /// </summary>
        public string? CallAddress { get; set; }

        /// <summary>
        /// 来电号码
        /// </summary>
        public string? FromPhone { get; set; }

        /// <summary>
        /// 转接号码（转接来源）
        /// </summary>
        public string? TransferPhone { get; set; }

        /// <summary>
        /// 来电/信人姓名
        /// </summary>
        public string? FromName { get; set; }

        /// <summary>
        /// 来电/信人性别
        /// </summary>
        public EGender FromGender { get; set; }

        /// <summary>
        /// 来电/信人身份
        /// </summary>
        public EIdentityType? IdentityType { get; set; }

        /// <summary>
        /// 证件类型
        /// </summary>
        public string? LicenceTypeCode { get; set; }

        public string? LicenceType { get; set; }

        /// <summary>
        /// 证件号码
        /// </summary>
        public string? LicenceNo { get; set; }

        /// <summary>
        /// 年龄段
        /// </summary>
        public string? AgeRangeCode { get; set; }

        public string? AgeRange { get; set; }

        /// <summary>
        /// 联系电话
        /// </summary>
        public string? Contact { get; set; }

        /// <summary>
        /// 是否接受短信，勾选校验手机号
        /// </summary>
        public bool AcceptSms { get; set; }

        /// <summary>
        /// 是否已发送短信
        /// </summary>
        public bool SmsSended { get; set; }

        /// <summary>
        /// 是否保密
        /// </summary>
        public bool IsSecret { get; set; }

        /// <summary>
        /// 工作单位(当“来电/信人身份”为“企业”时必填，其他情况非必填)
        /// </summary>
        public string? Company { get; set; }

        #endregion

        #region 诉求信息

        /// <summary>
        /// 受理类型
        /// </summary>
        public string? AcceptType { get; set; }
        public string? AcceptTypeCode { get; set; }

        /// <summary>
        /// 紧急程度
        /// </summary>
        public EEmergencyLevel EmergencyLevel { get; set; }

        public string Title { get; set; }

        #region 热点

        /// <summary>
        /// 热点
        /// </summary>
        public string? HotspotId { get; set; }
        //public string? HotspotCode { get; set; }
        public string? HotspotName { get; set; }
        public string? HotspotSpliceName { get; set; }

        /// <summary>
        /// 外部数据（为前端提供级联功能）
        /// </summary>
        public string? HotspotExternal { get; set; }

        #endregion

        #region 事件分类

        public string? EventCategoryId { get; set; }
        public string? EventCategoryName { get; set; }
        public string? EventCategorySpliceName { get; set; }
        public string? EventCategoryExternal { get; set; }

        #endregion

        /// <summary>
        /// 事发时间
        /// </summary>
        public DateTime? IncidentTime { get; set; }

        /// <summary>
        /// 事件目的
        /// </summary>
        public string? IncidentPurpose { get; set; }

        /// <summary>
        /// 重复工单Id
        /// </summary>
        public List<string>? DuplicateIds { get; set; }

        /// <summary>
        /// 推送分类
        /// </summary>
        public string? PushTypeCode { get; set; }

        public string? PushType { get; set; }

        /// <summary>
        /// 附件
        /// </summary>
        public List<string> Additions { get; set; } = new();

        /// <summary>
        /// 诉求内容
        /// </summary>
        public string Content { get; set; }

        #endregion

        /// <summary>
        /// 来源，区分省平台或110等其他平台同步过来的工单
        /// </summary>
        public ESource Source { get; set; } = ESource.Hotline;

        /// <summary>
        /// 扩展信息
        /// </summary>
        public OrderExtensionDto? OrderExtension { get; set; }

        /// <summary>
        /// 附件列表
        /// </summary>
        public List<FileDto> Files { get; set; } = new();

        /// <summary>
        /// 企业名称
        /// </summary>
        public string? EnterpriseName { get; set; }

        /// <summary>
        /// 企业Code
        /// </summary>
        public string? EnterpriseCode { get; set; }
        /// <summary>
        /// 专班名称
        /// </summary>
        public string? ZhuanBanMingCheng { get; set; }

        /// <summary>
        /// 专班Code
        /// </summary>
        public string? ZhuanBanCode { get; set; }

        #region  重复性事件
        public List<RepeatableEventDetailOpDto> RepeatableEventDetails { get; set; } = new();
        #endregion

        /// <summary>
        /// 是否为省工单
        /// </summary>
        public bool IsProvince { get; set; }

        /// <summary>
        /// 同步省工单编号和省上传下来的工单都用这个字段
        /// </summary>
        public string? ProvinceNo { get; set; }

        /// <summary>
        /// 省过期时间（省工单才有）
        /// </summary>
        public DateTime? ExpiredTimeProvince { get; set; }

        /// <summary>
        /// 外部工单唯一标识
        /// </summary>
        public string? ExternalId { get; set; }

        #region 期满时间

        /// <summary>
        /// 过期时间
        /// </summary>
        public DateTime? ExpiredTime { get; set; }

        /// <summary>
        /// 即将超期时间
        /// </summary>
        public DateTime? NearlyExpiredTime { get; set; }

        /// <summary>
        /// 办理时间限制(如：24小时、7个工作日)
        /// </summary>
        public string? TimeLimit { get; set; }

        public int? TimeLimitCount { get; set; }

        public ETimeType? TimeLimitUnit { get; set; }

        #endregion

        #region 市州工单互转
        /// <summary>
        /// 是否转发
        /// </summary>
        public bool? Transpond { get; set; }

        /// <summary>
        /// 转发市州Id
        /// </summary>
        public string? TranspondCityId { get; set; }

        /// <summary>
        /// 转发市州名称
        /// </summary>
        public string? TranspondCityName { get; set; }

        /// <summary>
        /// 转发市州Value
        /// </summary>
        public string? TranspondCityValue { get; set; }
        #endregion

        /// <summary>
        /// 是否行政执法类
        /// </summary>
        public bool? IsEnforcementOrder { get; set; }

        /// <summary>
        /// 形式主义工单
        /// </summary>
        public bool IsFormalistWorkOrder { get; set; }

        /// <summary>
        /// 敏感类工单
        /// </summary>
        public bool IsSensitiveWorkOrders { get; set; }

        /// <summary>
        /// 重点关注事件,保存前端选择，用于返回前端数据
        /// </summary>
        public string? FocusOnEvents { get; set; }

        /// <summary>
        /// 是否紧急
        /// </summary>
        public bool IsUrgent { get; set; }
    }

    public record CanLinkCallRecordOrderDto : PagedKeywordRequest
    {

    }

    /// <summary>
    /// 修改工单受理附件
    /// </summary>
    public class UpdateOrderFilesDto
    {
        public string Id { get; set; }

        public string OrderNo { get; set; }

        /// <summary>
        /// 附件
        /// </summary>
        public List<FileDto> Files { get; set; } = new List<FileDto>();
    }


    public record OrderCopyListDto : PagedKeywordRequest
    {
        public string OrderId { get; set; }
    }

    public class UnsignedOrderDto
    {

        public OrderDto Order { get; set; }

        public WorkflowStepDto WorkflowStep { get; set; }

        public decimal UnsignedTime { get; set; }
    }
}
